@@ -1,4 +1,5 @@
 #include "libretro.h"
+#include "libretro_core_options.h"
 #include <stdint.h>
 #include <string.h>
 #include <math.h>
@@ -19,13 +20,16 @@
 
 #include "../source/core/NstMachine.hpp"
 
-#define NST_VERSION "1.49"
+#include "nstdatabase.hpp"
+
+#define NST_VERSION "1.50"
 
 #define MIN(a,b)      ((a)<(b)?(a):(b))
 #define MAX(a,b)      ((a)>(b)?(a):(b))
 #define NES_NTSC_PAR ((Api::Video::Output::WIDTH - (overscan_h ? 16 : 0)) * (8.0 / 7.0)) / (Api::Video::Output::HEIGHT - (overscan_v ? 16 : 0))
 #define NES_PAL_PAR ((Api::Video::Output::WIDTH - (overscan_h ? 16 : 0)) * (2950000.0 / 2128137.0)) / (Api::Video::Output::HEIGHT - (overscan_v ? 16 : 0))
 #define NES_4_3_DAR (4.0 / 3.0);
+#define SAMPLERATE 48000
 
 using namespace Nes;
 
@@ -43,8 +47,8 @@ extern "C" void linearFree(void* mem);
 #endif
 static uint32_t* video_buffer = NULL;
 
-static int16_t audio_buffer[(48000 / 50)];
-static int16_t audio_stereo_buffer[2 * (48000 / 50)];
+static int16_t audio_buffer[(SAMPLERATE / 50)];
+static int16_t audio_stereo_buffer[2 * (SAMPLERATE / 50)];
 static Api::Emulator emulator;
 static Api::Machine *machine;
 static Api::Fds *fds;
@@ -58,6 +62,7 @@ static bool overscan_v;
 static bool overscan_h;
 static unsigned aspect_ratio_mode;
 static unsigned tpulse;
+static bool libretro_supports_bitmasks = false;
 
 int16_t video_width = Api::Video::Output::WIDTH;
 size_t pitch;
@@ -71,7 +76,6 @@ static Api::Machine::FavoredSystem favsystem;
 static void *sram;
 static unsigned long sram_size;
 static bool is_pal;
-static bool dbpresent;
 static byte custpal[64*3];
 static char slash;
 
@@ -223,32 +227,39 @@ void draw_crosshair(int x, int y)
 static void load_wav(const char* sampgame, Api::User::File& file)
 {
    char samp_path[292];
+   int length = 0;
+   int blockalign = 0;
+   int numchannels = 0;
+   int bitspersample = 0;
+   char fmt[4] = { 0x66, 0x6d, 0x74, 0x20};
+   char subchunk2id[4] = { 0x64, 0x61, 0x74, 0x61};
+   char *wavfile;
+   char *dataptr;
 
-   snprintf(samp_path, sizeof(samp_path), "%s%c%s%c%02d.wav", samp_dir, slash, sampgame, slash, file.GetId());
+   sprintf(samp_path, "%s%c%s%c%02d.wav", samp_dir, slash, sampgame, slash, file.GetId());
 
    std::ifstream samp_file(samp_path, std::ifstream::in|std::ifstream::binary);
 
    if (samp_file) {
 	   samp_file.seekg(0, samp_file.end);
-	   int length = samp_file.tellg();
+	   length = samp_file.tellg();
 	   samp_file.seekg(0, samp_file.beg);
-	   char wavfile[length];
+	   wavfile = (char*)malloc(length * sizeof(char));
 	   samp_file.read(wavfile, length);
 
 	   // Check to see if it has a valid header
-	   char fmt[4] = { 0x66, 0x6d, 0x74, 0x20};
-	   char subchunk2id[4] = { 0x64, 0x61, 0x74, 0x61};
 	   if (memcmp(&wavfile[0x00], "RIFF", 4) != 0) { return; }
 	   if (memcmp(&wavfile[0x08], "WAVE", 4) != 0) { return; }
 	   if (memcmp(&wavfile[0x0c], &fmt, 4) != 0) { return; }
 	   if (memcmp(&wavfile[0x24], &subchunk2id, 4) != 0) { return; }
 
 	   // Load the sample into the emulator
-	   char *dataptr = &wavfile[0x2c];
-	   int blockalign = wavfile[0x21] << 8 | wavfile[0x20];
-	   int numchannels = wavfile[0x17] << 8 | wavfile[0x16];
-	   int bitspersample = wavfile[0x23] << 8 | wavfile[0x22];
+	   dataptr = &wavfile[0x2c];
+	   blockalign = wavfile[0x21] << 8 | wavfile[0x20];
+	   numchannels = wavfile[0x17] << 8 | wavfile[0x16];
+	   bitspersample = wavfile[0x23] << 8 | wavfile[0x22];
 	   file.SetSampleContent(dataptr, (length - 44) / blockalign, 0, bitspersample, 44100);
+	   free(wavfile);
    }
 }
 
@@ -338,12 +349,15 @@ void retro_init(void)
    else
       log_cb = NULL;
 
+   if (environ_cb(RETRO_ENVIRONMENT_GET_INPUT_BITMASKS, NULL))
+      libretro_supports_bitmasks = true;
 
    check_system_specs();
 }
 
 void retro_deinit(void)
 {
+   libretro_supports_bitmasks = false;
 }
 
 unsigned retro_api_version(void)
@@ -388,7 +402,7 @@ double get_aspect_ratio(void)
 
 void retro_get_system_av_info(struct retro_system_av_info *info)
 {
-   const retro_system_timing timing = { is_pal ? 50.0 : 60.0, 48000.0 };
+   const retro_system_timing timing = { is_pal ? 50.0 : 60.0, SAMPLERATE };
    info->timing = timing;
 
    // It's better if the size is based on NTSC_WIDTH if the filter is on
@@ -406,25 +420,7 @@ void retro_get_system_av_info(struct retro_system_av_info *info)
 void retro_set_environment(retro_environment_t cb)
 {
    environ_cb = cb;
-
-   static const struct retro_variable vars[] = {
-      { "nestopia_blargg_ntsc_filter", "Blargg NTSC filter; disabled|composite|svideo|rgb|monochrome" },
-      { "nestopia_palette", "Palette; cxa2025as|consumer|canonical|alternative|rgb|pal|composite-direct-fbx|pvm-style-d93-fbx|ntsc-hardware-fbx|nes-classic-fbx-fs|raw|custom" },
-      { "nestopia_nospritelimit", "Remove 8-sprites-per-scanline hardware limit; disabled|enabled" },
-      { "nestopia_overclock", "CPU Speed (Overclock); 1x|2x" },
-	  { "nestopia_select_adapter", "4 Player Adapter; auto|ntsc|famicom" },
-      { "nestopia_fds_auto_insert", "Automatically insert first FDS disk on reset; enabled|disabled" },
-      { "nestopia_overscan_v", "Mask Overscan (Vertical); enabled|disabled" },
-      { "nestopia_overscan_h", "Mask Overscan (Horizontal); disabled|enabled" },
-      { "nestopia_aspect" ,  "Preferred aspect ratio; auto|ntsc|pal|4:3" },
-      { "nestopia_genie_distortion", "Game Genie Sound Distortion; disabled|enabled" },
-      { "nestopia_favored_system", "Favored System; auto|ntsc|pal|famicom|dendy" },
-      { "nestopia_ram_power_state", "RAM Power-on State; 0x00|0xFF|random" },
-      { "nestopia_turbo_pulse", "Turbo Pulse Speed; 2|3|4|5|6|7|8|9" },
-      { NULL, NULL },
-   };
-
-   cb(RETRO_ENVIRONMENT_SET_VARIABLES, (void*)vars);
+   libretro_set_core_options(environ_cb);
 }
 
 void retro_set_audio_sample(retro_audio_sample_t cb)
@@ -470,9 +466,11 @@ typedef struct
    unsigned nes;
 } keymap;
 
-static const keymap bindmap[] = {
+static keymap bindmap_default[] = {
    { RETRO_DEVICE_ID_JOYPAD_A, Core::Input::Controllers::Pad::A },
    { RETRO_DEVICE_ID_JOYPAD_B, Core::Input::Controllers::Pad::B },
+   { RETRO_DEVICE_ID_JOYPAD_X, Core::Input::Controllers::Pad::A },
+   { RETRO_DEVICE_ID_JOYPAD_Y, Core::Input::Controllers::Pad::B },
    { RETRO_DEVICE_ID_JOYPAD_SELECT, Core::Input::Controllers::Pad::SELECT },
    { RETRO_DEVICE_ID_JOYPAD_START, Core::Input::Controllers::Pad::START },
    { RETRO_DEVICE_ID_JOYPAD_UP, Core::Input::Controllers::Pad::UP },
@@ -481,7 +479,22 @@ static const keymap bindmap[] = {
    { RETRO_DEVICE_ID_JOYPAD_RIGHT, Core::Input::Controllers::Pad::RIGHT },
 };
 
-static void update_input()
+static keymap bindmap_shifted[] = {
+   { RETRO_DEVICE_ID_JOYPAD_B, Core::Input::Controllers::Pad::A },
+   { RETRO_DEVICE_ID_JOYPAD_Y, Core::Input::Controllers::Pad::B },
+   { RETRO_DEVICE_ID_JOYPAD_A, Core::Input::Controllers::Pad::A },
+   { RETRO_DEVICE_ID_JOYPAD_X, Core::Input::Controllers::Pad::B },
+   { RETRO_DEVICE_ID_JOYPAD_SELECT, Core::Input::Controllers::Pad::SELECT },
+   { RETRO_DEVICE_ID_JOYPAD_START, Core::Input::Controllers::Pad::START },
+   { RETRO_DEVICE_ID_JOYPAD_UP, Core::Input::Controllers::Pad::UP },
+   { RETRO_DEVICE_ID_JOYPAD_DOWN, Core::Input::Controllers::Pad::DOWN },
+   { RETRO_DEVICE_ID_JOYPAD_LEFT, Core::Input::Controllers::Pad::LEFT },
+   { RETRO_DEVICE_ID_JOYPAD_RIGHT, Core::Input::Controllers::Pad::RIGHT },
+};
+
+static keymap *bindmap = bindmap_default;
+
+static void update_input(bool supports_bitmasks)
 {
    input_poll_cb();
    input->pad[0].buttons = 0;
@@ -491,8 +504,9 @@ static void update_input()
    input->pad[1].mic = 0;
    input->zapper.fire = 0;
    input->vsSystem.insertCoin = 0;
-   
-   if (Api::Input(emulator).GetConnectedController(1) == 5) {
+
+   if (Api::Input(emulator).GetConnectedController(1) == 5)
+   {
       static int zapx = overscan_h ? 8 : 0; 
       static int zapy = overscan_v ? 8 : 0;
       int min_x = overscan_h ? 8 : 0;
@@ -527,40 +541,79 @@ static void update_input()
          input->zapper.y = zapy;
          input->zapper.fire = 1;
       }
-      
-      if (input_state_cb(1, RETRO_DEVICE_LIGHTGUN, 0, RETRO_DEVICE_ID_LIGHTGUN_TURBO)) {
+
+      if (input_state_cb(1, RETRO_DEVICE_LIGHTGUN, 0, RETRO_DEVICE_ID_LIGHTGUN_RELOAD)) {
          input->zapper.x = ~1U;
          input->zapper.fire = 1;
       }
    }
    
    static unsigned tstate = 2;
-   
-   for (unsigned p = 0; p < 4; p++)
-      for (unsigned bind = 0; bind < sizeof(bindmap) / sizeof(bindmap[0]); bind++)
+   bool pressed_l3        = false;
+   bool pressed_l2        = false;
+   bool pressed_r2        = false;
+   bool pressed_l         = false;
+   bool pressed_r         = false;
+
+   if (supports_bitmasks)
+   {
+      int16_t ret[4];
+      /* Player 0 needs some extra checks */
+      ret[0]              = input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_MASK);
+      ret[1]              = input_state_cb(1, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_MASK);
+      ret[2]              = input_state_cb(2, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_MASK);
+      ret[3]              = input_state_cb(3, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_MASK);
+      pressed_l3          = ret[0] & (1 << RETRO_DEVICE_ID_JOYPAD_L3);
+      pressed_l2          = ret[0] & (1 << RETRO_DEVICE_ID_JOYPAD_L2);
+      pressed_r2          = ret[0] & (1 << RETRO_DEVICE_ID_JOYPAD_R2);
+      pressed_l           = ret[0] & (1 << RETRO_DEVICE_ID_JOYPAD_L );
+      pressed_r           = ret[0] & (1 << RETRO_DEVICE_ID_JOYPAD_R );
+
+      for (unsigned p = 0; p < 4; p++)
       {
-         input->pad[p].buttons |= input_state_cb(p, RETRO_DEVICE_JOYPAD, 0, bindmap[bind].retro) ? bindmap[bind].nes : 0;
-         if (input_state_cb(p, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_X))
+         for (unsigned bind = 0; bind < sizeof(bindmap_default) / sizeof(bindmap[0]); bind++)
+            input->pad[p].buttons |= (ret[p] & (1 << bindmap[bind].retro)) ? bindmap[bind].nes : 0;
+         if (ret[p] & (1 << bindmap[2].retro))
             tstate ? input->pad[p].buttons &= ~Core::Input::Controllers::Pad::A : input->pad[p].buttons |= Core::Input::Controllers::Pad::A;
-         if (input_state_cb(p, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_Y))
+         if (ret[p] & (1 << bindmap[3].retro))
             tstate ? input->pad[p].buttons &= ~Core::Input::Controllers::Pad::B : input->pad[p].buttons |= Core::Input::Controllers::Pad::B;
       }
+   }
+   else
+   {
+      pressed_l3          = input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L3);
+      pressed_l2          = input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L2);
+      pressed_r2          = input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_R2);
+      pressed_l           = input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L);
+      pressed_r           = input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_R);
+
+      for (unsigned p = 0; p < 4; p++)
+      {
+         for (unsigned bind = 0; bind < sizeof(bindmap_default) / sizeof(bindmap[0]); bind++)
+            input->pad[p].buttons |= input_state_cb(p, RETRO_DEVICE_JOYPAD, 0, bindmap[bind].retro) ? bindmap[bind].nes : 0;
+         if (input_state_cb(p, RETRO_DEVICE_JOYPAD, 0, bindmap[2].retro))
+            tstate ? input->pad[p].buttons &= ~Core::Input::Controllers::Pad::A : input->pad[p].buttons |= Core::Input::Controllers::Pad::A;
+         if (input_state_cb(p, RETRO_DEVICE_JOYPAD, 0, bindmap[3].retro))
+            tstate ? input->pad[p].buttons &= ~Core::Input::Controllers::Pad::B : input->pad[p].buttons |= Core::Input::Controllers::Pad::B;
+      }
+   }
       
    if (tstate) tstate--; else tstate = tpulse;
    
-   if (input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L3))
+   if (pressed_l3)
       input->pad[1].mic |= 0x04;
    
-   if (input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L2))
+   if (pressed_l2)
       input->vsSystem.insertCoin |= Core::Input::Controllers::VsSystem::COIN_1;
       
-   if (input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_R2))
+   if (pressed_r2)
       input->vsSystem.insertCoin |= Core::Input::Controllers::VsSystem::COIN_2;
       
    if (machine->Is(Nes::Api::Machine::DISK))
    {
-      bool curL = input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_L);
+      bool curL         = pressed_l;
       static bool prevL = false;
+
       if (curL && !prevL)
       {
          if (!fds->IsAnyDiskInserted())
@@ -570,8 +623,9 @@ static void update_input()
       }
       prevL = curL;
       
-      bool curR = input_state_cb(0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_R);
+      bool curR         = pressed_r;
       static bool prevR = false;
+
       if (curR && !prevR && (fds->GetNumDisks() > 1))
       {
          int currdisk = fds->GetCurrentDisk();
@@ -594,6 +648,16 @@ static void check_variables(void)
    Api::Machine machine(emulator);
    Api::Video::RenderState::Filter filter;
 
+   var.key = "nestopia_button_shift";
+   
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var))
+   {
+      if (strcmp(var.value, "disabled") == 0)
+         bindmap = bindmap_default;
+      else if (strcmp(var.value, "enabled") == 0)
+         bindmap = bindmap_shifted;
+   }
+   
    var.key = "nestopia_favored_system";
    is_pal = false;
 
@@ -601,20 +665,17 @@ static void check_variables(void)
    {
       if (strcmp(var.value, "auto") == 0)
       {
-         if (dbpresent)
+         machine.SetMode(machine.GetDesiredMode());
+         if (machine.GetMode() == Api::Machine::PAL)
          {
-            machine.SetMode(machine.GetDesiredMode());
-            if (machine.GetMode() == Api::Machine::PAL)
-            {
-               is_pal = true;
-               favsystem = Api::Machine::FAVORED_NES_PAL;
-               machine.SetMode(Api::Machine::PAL);
-            }
-            else
-            {
-               favsystem = Api::Machine::FAVORED_NES_NTSC;
-               machine.SetMode(Api::Machine::NTSC);
-            }
+            is_pal = true;
+            favsystem = Api::Machine::FAVORED_NES_PAL;
+            machine.SetMode(Api::Machine::PAL);
+         }
+         else
+         {
+            favsystem = Api::Machine::FAVORED_NES_NTSC;
+            machine.SetMode(Api::Machine::NTSC);
          }
       }
       else if (strcmp(var.value, "ntsc") == 0)
@@ -646,7 +707,7 @@ static void check_variables(void)
       }
    }
    if (audio) delete audio;
-   audio = new Api::Sound::Output(audio_buffer, is_pal ? 48000 / 50 : 48000 / 60);
+   audio = new Api::Sound::Output(audio_buffer, is_pal ? SAMPLERATE / 50 : SAMPLERATE / 60);
 
    var.key = "nestopia_genie_distortion";
 
@@ -856,12 +917,9 @@ static void check_variables(void)
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
    { 
 	   if (!strcmp(var.value, "auto")) {
-		   if (dbpresent)
-		   {
-			   Api::Input(emulator).AutoSelectController(2);
-			   Api::Input(emulator).AutoSelectController(3);
-			   Api::Input(emulator).AutoSelectAdapter();
-		   }
+		   Api::Input(emulator).AutoSelectController(2);
+		   Api::Input(emulator).AutoSelectController(3);
+		   Api::Input(emulator).AutoSelectAdapter();
 	   }
 	   else if (!strcmp(var.value, "ntsc")) {
 		   Api::Input(emulator).ConnectController(2, Api::Input::PAD3);
@@ -890,7 +948,7 @@ static void check_variables(void)
    renderState.bits.mask.g = 0x0000ff00;
    renderState.bits.mask.b = 0x000000ff;
    if (NES_FAILED(video.SetRenderState( renderState )) && log_cb)
-      log_cb(RETRO_LOG_INFO, "Nestopia core rejected render state\n");;
+      log_cb(RETRO_LOG_WARN, "Nestopia core rejected render state\n");;
 
    retro_get_system_av_info(&av_info);
    environ_cb(RETRO_ENVIRONMENT_SET_GEOMETRY, &av_info);
@@ -899,13 +957,13 @@ static void check_variables(void)
 
 void retro_run(void)
 {
-   update_input();
+   update_input(libretro_supports_bitmasks);
    emulator.Execute(video, audio, input);
 
    if (Api::Input(emulator).GetConnectedController(1) == 5)
       draw_crosshair(crossx, crossy);
    
-   unsigned frames = is_pal ? 48000 / 50 : 48000 / 60;
+   unsigned frames = is_pal ? SAMPLERATE / 50 : SAMPLERATE / 60;
    for (unsigned i = 0; i < frames; i++)
       audio_stereo_buffer[(i << 1) + 0] = audio_stereo_buffer[(i << 1) + 1] = audio_buffer[i];
    audio_batch_cb(audio_stereo_buffer, frames);
@@ -1063,37 +1121,24 @@ bool retro_load_game(const struct retro_game_info *info)
    {
       custompalette->read((char*)custpal, sizeof(custpal));
       if (log_cb)
-         log_cb(RETRO_LOG_WARN, "custom.pal loaded from system directory.\n");
+         log_cb(RETRO_LOG_INFO, "custom.pal loaded from system directory.\n");
    }
    else
    {
       memcpy(custpal, cxa2025as_palette, sizeof(custpal));
       if (log_cb)
-         log_cb(RETRO_LOG_WARN, "custom.pal not found in system directory.\n");
+         log_cb(RETRO_LOG_INFO, "custom.pal not found in system directory.\n");
    }
    delete custompalette;
    
-   sprintf(db_path, "%s%cNstDatabase.xml", dir, slash);
-
-   if (log_cb)
-      log_cb(RETRO_LOG_INFO, "NstDatabase.xml path: %s\n", db_path);
-   
    Api::Cartridge::Database database(emulator);
-   std::ifstream *db_file = new std::ifstream(db_path, std::ifstream::in|std::ifstream::binary);
    
-   if (db_file->is_open())
-   {
-      database.Load(*db_file);
-      database.Enable(true);
-      dbpresent = true;
-   }
-   else
-   {
-      if (log_cb)
-         log_cb(RETRO_LOG_WARN, "NstDatabase.xml required to detect region and some mappers.\n");
-      delete db_file;
-      dbpresent = false;
-   }
+   size_t db_size = sizeof(nst_db_xml)/sizeof(unsigned char);
+   std::string db_buf((const char*)nst_db_xml, db_size);
+   std::istringstream *db_file = new std::istringstream(db_buf);
+   
+   database.Load(*db_file);
+   database.Enable(true);
    
    if (info->path != NULL)
    {
@@ -1171,20 +1216,11 @@ bool retro_load_game(const struct retro_game_info *info)
 
    Api::Sound isound(emulator);
    isound.SetSampleBits(16);
-   isound.SetSampleRate(48000);
+   isound.SetSampleRate(SAMPLERATE);
    isound.SetSpeaker(Api::Sound::SPEAKER_MONO);
 
-   if (dbpresent)
-   {
-      Api::Input(emulator).AutoSelectController(0);
-      Api::Input(emulator).AutoSelectController(1);
-   }
-   else
-   {
-      Api::Input(emulator).ConnectController(0, Api::Input::PAD1);
-      Api::Input(emulator).ConnectController(1, Api::Input::PAD2);
-      //Api::Input(emulator).ConnectController(1, Api::Input::ZAPPER);
-   }
+   Api::Input(emulator).AutoSelectController(0);
+   Api::Input(emulator).AutoSelectController(1);
 
    machine->Power(true);
 
