@@ -23,6 +23,7 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 
 #include <cstring>
+#include <cstdlib>
 #include "NstCpu.hpp"
 #include "NstHook.hpp"
 #include "NstState.hpp"
@@ -166,6 +167,11 @@ namespace Nes
 			Reset( false, true );
 		}
 
+		void Cpu::SetRamPowerState(uint powerstate)
+		{
+			ram.powerstate = powerstate;
+		}
+
 		void Cpu::Reset(bool hard)
 		{
 			Reset( true, hard );
@@ -198,6 +204,8 @@ namespace Nes
 			ticks   = 0;
 			logged  = 0;
 
+			cpuOverclocking = false;
+
 			pc = RESET_VECTOR;
 
 			cycles.count  = 0;
@@ -767,18 +775,10 @@ namespace Nes
 
 		void Cpu::Ram::Reset(const CpuModel model)
 		{
-			if (model == CPU_DENDY)
-			{
-				std::memset( mem, 0x00, sizeof(mem) );
-			}
-			else
-			{
-				std::memset( mem, 0xFF, sizeof(mem) );
-
-				mem[0x08] = 0xF7;
-				mem[0x09] = 0xEF;
-				mem[0x0A] = 0xDF;
-				mem[0x0F] = 0xBF;
+			switch (powerstate) {
+				case 1: std::memset( mem, 0xFF, sizeof(mem) ); break;
+				case 2: std::memset( mem, byte(std::rand()), sizeof(mem) ); break;
+				default: std::memset( mem, 0x00, sizeof(mem) ); break;
 			}
 		}
 
@@ -1901,6 +1901,18 @@ namespace Nes
 
 			if (interrupt.irqClock != CYCLE_MAX)
 				interrupt.irqClock = (interrupt.irqClock > cycles.frame ? interrupt.irqClock - cycles.frame : 0);
+
+			if (cpuOverclocking)
+			{
+				uint startCycle = cycles.count;
+				uint lastCycle = cycles.count + extraCycles;
+				do
+				{
+					ExecuteOp();
+				}
+				while (cycles.count < extraCycles);
+				cycles.count = startCycle;
+			}
 		}
 
 		void Cpu::Clock()
