@@ -162,12 +162,13 @@ namespace Nes
 
 		Fds::Fds(Context& context)
 		:
-		Image   (DISK),
-		disks   (context.stream),
-		adapter (context.cpu,disks.sides),
-		cpu     (context.cpu),
-		ppu     (context.ppu),
-		sound   (context.apu)
+		Image         (DISK),
+		disks         (context.stream),
+		adapter       (context.cpu,disks.sides),
+		cpu           (context.cpu),
+		ppu           (context.ppu),
+		sound         (context.apu),
+		favoredSystem (context.favoredSystem)
 		{
 			if (!bios.Available())
 				throw RESULT_ERR_MISSING_BIOS;
@@ -270,6 +271,16 @@ namespace Nes
 
 				return SYSTEM_FAMICOM;
 			}
+			else if ((region == REGION_PAL) && (favoredSystem == FAVORED_DENDY))
+			{
+				if (cpu)
+					*cpu = CPU_DENDY;
+
+				if (ppu)
+					*ppu = PPU_DENDY;
+
+				return SYSTEM_DENDY;
+			}
 			else
 			{
 				return Image::GetDesiredSystem( region, cpu, ppu );
@@ -962,7 +973,7 @@ namespace Nes
 				count = 0;
 				status |= uint(STATUS_UNREADY);
 			}
-			else if (!(reg & CTRL_STOP | count) && io)
+			else if (!((reg & CTRL_STOP) | count) && io)
 			{
 				count = CLK_MOTOR;
 				headPos = 0;
@@ -1284,7 +1295,7 @@ namespace Nes
 					State::Loader::Data<16> data( state );
 
 					unit.drive.ctrl = data[0];
-					unit.drive.status = data[1] & (Unit::Drive::STATUS_EJECTED|Unit::Drive::STATUS_UNREADY|Unit::Drive::STATUS_PROTECTED) | OPEN_BUS;
+					unit.drive.status = (data[1] & (Unit::Drive::STATUS_EJECTED|Unit::Drive::STATUS_UNREADY|Unit::Drive::STATUS_PROTECTED)) | OPEN_BUS;
 					unit.drive.in = data[2] | (data[15] << 8 & 0x100);
 					unit.drive.out = data[3];
 					unit.drive.headPos = data[4] | data[5] << 8;
