@@ -414,6 +414,9 @@ namespace Nes
 			Chr chr;
 			Nmt nmt;
 			int scanline;
+			int scanline_sleep;
+			int ssleep;
+			bool overclocked;
 
 			PpuModel model;
 			Hook hActiveHook;
@@ -540,6 +543,16 @@ namespace Nes
 			{
 				return oam.spriteLimit;
 			}
+
+			bool GetOverclockState() const
+			{
+				return overclocked;
+			}
+
+			void SetOverclockState(bool overclock2x)
+			{
+				overclocked = overclock2x;
+			}
 		};
 	}
 }
