@@ -208,6 +208,7 @@ namespace Nes
 						STD_UN1ROM                 = MakeId<   94,  128,    0,  0,  0, CRM_8,  NMT_X,  0 >::ID,
 						STD_UOROM                  = MakeId<    2,  256,    0,  0,  0, CRM_8,  NMT_X,  0 >::ID,
 						STD_UXROM                  = MakeId<    2,  256,    0,  0,  0, CRM_8,  NMT_X,  1 >::ID,
+						STD_UNROM512               = MakeId<   30,  512,    0,  0,  0, CRM_32, NMT_X,  0 >::ID,
 						// Discrete Logic
 						DISCRETE_74_377            = MakeId<   11,  128,  128,  0,  0, CRM_0,  NMT_X,  0 >::ID,
 						DISCRETE_74_139_74         = MakeId<   87,   32,   32,  0,  0, CRM_0,  NMT_X,  4 >::ID,
@@ -232,6 +233,8 @@ namespace Nes
 						CUSTOM_ZZ                  = MakeId<   37,  256,  256,  0,  0, CRM_0,  NMT_V,  0 >::ID,
 						// Acclaim
 						ACCLAIM_MCACC              = MakeId<    4,  512,  256,  0,  0, CRM_0,  NMT_X,  1 >::ID,
+						// Action 53
+						ACTION53                   = MakeId<   28, 1024,    0,  0,  0, CRM_32, NMT_X,  0 >::ID,
 						// Active Enterprises
 						AE_STD                     = MakeId<  228, 2048,  512,  0,  0, CRM_0,  NMT_V,  0 >::ID,
 						// AGCI
@@ -355,6 +358,8 @@ namespace Nes
 						HES_STD                    = MakeId<  113,  256,  128,  0,  0, CRM_0,  NMT_H,  0 >::ID,
 						// Hosenkan
 						HOSENKAN_STD               = MakeId<  182,  512,  256,  0,  0, CRM_0,  NMT_X,  0 >::ID,
+						// Infinite NES Lives
+						INLNSF                     = MakeId<   31, 1024,    8,  0,  0, CRM_0,  NMT_X,  0 >::ID,
 						// Irem
 						IREM_G101A_0               = MakeId<   32,  256,  128,  0,  0, CRM_0,  NMT_V,  0 >::ID,
 						IREM_G101A_1               = MakeId<   32,  256,  128,  8,  0, CRM_0,  NMT_V,  0 >::ID,
@@ -414,6 +419,7 @@ namespace Nes
 						// Kaiser
 						KAISER_KS202               = MakeId<   56,  256,  128,  8,  0, CRM_0,  NMT_V,  0 >::ID,
 						KAISER_KS7022              = MakeId<  175,  256,  128,  0,  0, CRM_0,  NMT_V,  0 >::ID,
+						KAISER_KS7031              = MakeId<  NMPR, 128,    0,  0, 16, CRM_8,  NMT_V,  0 >::ID,
 						KAISER_KS7032              = MakeId<  142,  128,    0,  0,  0, CRM_8,  NMT_X,  0 >::ID,
 						KAISER_KS7058              = MakeId<  171,   32,   32,  0,  0, CRM_0,  NMT_X,  0 >::ID,
 						// Kasing
@@ -551,6 +557,7 @@ namespace Nes
 						UNL_UXROM_M5               = MakeId<  180, 4096,    8,  8,  0, CRM_0,  NMT_X,  0 >::ID,
 						UNL_TRXROM                 = MakeId<    4,  512,  256,  8,  0, CRM_0,  NMT_4,  0 >::ID,
 						UNL_XZY                    = MakeId<  176, 1024,  256,  8,  0, CRM_0,  NMT_X,  0 >::ID,
+						UNL_MMC3BIGCHRRAM          = MakeId<    4,  512,    0,  0,  0, CRM_32, NMT_X,  0 >::ID,
 						// Waixing
 						WAIXING_PS2_0              = MakeId<   15, 1024,    0,  0,  0, CRM_8,  NMT_V,  0 >::ID,
 						WAIXING_PS2_1              = MakeId<   15, 1024,    0,  8,  0, CRM_8,  NMT_V,  0 >::ID,
@@ -646,7 +653,7 @@ namespace Nes
 					Context(Cpu*,Apu*,Ppu*,Ram&,Ram&,const Ram&,Type::Nmt,bool,bool,Chips&);
 
 					bool DetectBoard(wcstring,dword);
-					bool DetectBoard(byte,dword,bool,byte);
+					bool DetectBoard(byte,byte,dword,dword,bool);
 
 					cstring name;
 					Type type;
